/*
 
    MiniM
    C# example how to call MiniM Server Connect
 
    This demo class MiniMExamples.MiniMSC    
    demonstrates how to declare and call minimsc.dll functions in C#

 
    Demo only. To use minimsc.dll inside your project write class 
    using your project coding and error handling conventions.
 
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace MiniMExamples
{
    class MiniMSC
    {
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
        public struct MINIMSTR
        {
            public const int MINIMSTR_MAX = 32 * 1024 - 1;
            public short len;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MINIMSTR_MAX)]
            public string data;
        }

        public delegate void OutputProc(IntPtr MNMConnect, ref MINIMSTR Value);

        public delegate void GroupReadProc(IntPtr MNMConnect, ref MINIMSTR Value);

        public delegate void CallbackProc( IntPtr MNMConnect, ref MINIMSTR Command, 
            ref MINIMSTR Answer);

        public static void StringToMINIMSTR(string s, ref MINIMSTR str)
        {
            if (s.Length <= MINIMSTR.MINIMSTR_MAX)
            {
                str.len = (short)s.Length;
                str.data = s;
            }
            else
            {
                str.len = (short)MINIMSTR.MINIMSTR_MAX;
                str.data = s.Substring( 0, str.len);
            }
            return;
        }

        public static string MINIMSTRToString( ref MINIMSTR str)
        {
            StringBuilder s = new StringBuilder();
            s.Append(str.data, 0, str.len);
            return s.ToString();
        }

        public const int DefaultPort = 5000;
        public const string DefaultDatabase = "USER";
        public const string DefaultServer = "localhost";

        public int Port;
        public string Server;
        public string Database;

        protected IntPtr MNMConnect;
        protected OutputProc Proc;
        protected GroupReadProc grProc;
        protected CallbackProc cbProc;

        public MiniMSC()
        {
            Port = DefaultPort;
            Server = DefaultServer;
            Database = DefaultDatabase;
            // MNMConnect = 0;
            Proc = null;
            grProc = null;
            cbProc = null;
        }

        [DllImport("minimsc.dll", CallingConvention = CallingConvention.StdCall,
         EntryPoint = "MNMCreateConnect", ExactSpelling = true, CharSet = CharSet.Ansi)]
        public static extern IntPtr MNMCreateConnect( string server,
            int port, string database);

        public bool CreateConnect()
        {
            MNMConnect = MNMCreateConnect( Server, Port, Database);
            return !MNMConnect.Equals( 0);
        }

        [DllImport("minimsc.dll", CallingConvention = CallingConvention.StdCall,
        EntryPoint = "MNMConnectOpen", ExactSpelling = true, CharSet = CharSet.Ansi)]
        public static extern int MNMConnectOpen(IntPtr Connect);

        public bool ConnectOpen()
        {
            return MNMConnectOpen(MNMConnect) != 0;
        }

        [DllImport("minimsc.dll", CallingConvention = CallingConvention.StdCall,
        EntryPoint = "MNMGetLastError", ExactSpelling = true, CharSet = CharSet.Ansi)]
        public static extern void MNMGetLastError( IntPtr Connect,
            ref MINIMSTR pError);

        public string GetLastError()
        {
            MINIMSTR Error = new MINIMSTR();
            MNMGetLastError( MNMConnect, ref Error);
            return MINIMSTRToString( ref Error);
        }

        [DllImport("minimsc.dll", CallingConvention = CallingConvention.StdCall,
        EntryPoint = "MNMRead", ExactSpelling = true, CharSet = CharSet.Ansi)]
        public static extern int MNMRead(IntPtr Connect,
            ref MINIMSTR Expression, ref MINIMSTR Result);

        public int Read( ref MINIMSTR Expression, ref MINIMSTR Result)
        {
            return MNMRead( MNMConnect, ref Expression, ref Result);
        }

        [DllImport("minimsc.dll", CallingConvention = CallingConvention.StdCall,
        EntryPoint = "MNMExecute", ExactSpelling = true, CharSet = CharSet.Ansi)]
        public static extern int MNMExecute(IntPtr Connect,
            ref MINIMSTR Commands);

        public int Execute(ref MINIMSTR Commands)
        {
            return MNMExecute( MNMConnect, ref Commands);
        }

        [DllImport("minimsc.dll", CallingConvention = CallingConvention.StdCall,
        EntryPoint = "MNMListLength", ExactSpelling = true, CharSet = CharSet.Ansi)]
        public static extern int MNMListLength(ref MINIMSTR List);

        [DllImport("minimsc.dll", CallingConvention = CallingConvention.StdCall,
        EntryPoint = "MNMListGet", ExactSpelling = true, CharSet = CharSet.Ansi)]
        public static extern int MNMListGet( ref MINIMSTR List, int pos, 
            ref MINIMSTR Element);

        [DllImport("minimsc.dll", CallingConvention = CallingConvention.StdCall,
        EntryPoint = "MNMListSet", ExactSpelling = true, CharSet = CharSet.Ansi)]
        public static extern int MNMListSet(ref MINIMSTR List, int pos,
            ref MINIMSTR Element);

        [DllImport("minimsc.dll", CallingConvention = CallingConvention.StdCall,
        EntryPoint = "MNMText", ExactSpelling = true, CharSet = CharSet.Ansi)]
        public static extern int MNMText(ref MINIMSTR Source, ref MINIMSTR Target);

        [DllImport("minimsc.dll", CallingConvention = CallingConvention.StdCall,
        EntryPoint = "MNMConnectClose", ExactSpelling = true, CharSet = CharSet.Ansi)]
        public static extern int MNMConnectClose(IntPtr Connect);

        public int ConnectClose()
        {
            int ret = MNMConnectClose(MNMConnect);
            return ret;
        }

        [DllImport("minimsc.dll", CallingConvention = CallingConvention.StdCall,
        EntryPoint = "MNMDestroyConnect", ExactSpelling = true, CharSet = CharSet.Ansi)]
        public static extern void MNMDestroyConnect(IntPtr Connect);

        public void DestroyConnect()
        {
            MNMDestroyConnect(MNMConnect);
            return;
        }

        [DllImport("minimsc.dll", CallingConvention = CallingConvention.StdCall,
        EntryPoint = "MNMSetCallback", ExactSpelling = true, CharSet = CharSet.Ansi)]
        public static extern void MNMSetCallback(IntPtr Connect, CallbackProc Proc);

        public void SetCallback(CallbackProc _Proc)
        {
            cbProc = _Proc;
            MNMSetCallback(MNMConnect, cbProc);
            return;
        }

        [DllImport("minimsc.dll", CallingConvention = CallingConvention.StdCall,
        EntryPoint = "MNMSetGroupRead", ExactSpelling = true, CharSet = CharSet.Ansi)]
        public static extern void MNMSetGroupRead(IntPtr Connect, GroupReadProc Proc);

        public void SetGroupRead(GroupReadProc _Proc)
        {
            grProc = _Proc;
            MNMSetGroupRead(MNMConnect, grProc);
            return;
        }

        [DllImport("minimsc.dll", CallingConvention = CallingConvention.StdCall,
        EntryPoint = "MNMSetOutput", ExactSpelling = true, CharSet = CharSet.Ansi)]
        public static extern void MNMSetOutput(IntPtr Connect, OutputProc Proc);

        public void SetOutput(OutputProc _Proc)
        {
            Proc = _Proc;
            MNMSetOutput( MNMConnect, Proc);
            return;
        }

        [DllImport("minimsc.dll", CallingConvention = CallingConvention.StdCall,
        EntryPoint = "MNMExecuteOutput", ExactSpelling = true, CharSet = CharSet.Ansi)]
        public static extern int MNMExecuteOutput(IntPtr Connect,
            ref MINIMSTR Commands);

        public int ExecuteOutput(ref MINIMSTR Commands)
        {
            return MNMExecuteOutput(MNMConnect, ref Commands);
        }

        [DllImport("minimsc.dll", CallingConvention = CallingConvention.StdCall,
        EntryPoint = "MNMWrite", ExactSpelling = true, CharSet = CharSet.Ansi)]
        public static extern int MNMWrite(IntPtr Connect,
            ref MINIMSTR VarName, ref MINIMSTR VarValue);

        public int Write(ref MINIMSTR VarName, ref MINIMSTR VarValue)
        {
            return MNMWrite(MNMConnect, ref VarName, ref VarValue);
        }

        [DllImport("minimsc.dll", CallingConvention = CallingConvention.StdCall,
        EntryPoint = "MNMKill", ExactSpelling = true, CharSet = CharSet.Ansi)]
        public static extern int MNMKill(IntPtr Connect,
            ref MINIMSTR VarName);

        public int Kill(ref MINIMSTR VarName)
        {
            return MNMKill(MNMConnect, ref VarName);
        }
        
    }
}
