/*

    MiniM

    C++ example to call MiniM Server Connect

*/

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>

#include "minimsc.h"

// this library should be created using your compiler from minimsc.dll
#pragma link "minimsc.lib"

void HandleMNMError( HMNMConnect Connect)
{
  MINIMSTR err;
  MNMGetLastError( Connect, &err);
  printf( "MiniM error:\n%.*s\n", err.len, err.data);
  return;
};

int main(int argc, char* argv[])
{
  printf( "C++ example how to call MiniM Server Connect\n");

  // create connect to MiniM
  HMNMConnect Connect = MNMCreateConnect( "localhost", 5000, "user");

  // activate connection
  int ret = MNMConnectOpen( Connect);
  if( !ret)
  {
    HandleMNMError( Connect);
    return 0;
  }

  // read version info
  MINIMSTR expr;
  MINIMSTR value;
  expr.len = sprintf( expr.data, "$zversion");
  ret = MNMRead( Connect, &expr, &value);
  if( !ret)
  {
    HandleMNMError( Connect);
    return 0;
  }

  // print result
  printf( "MiniM version: %.*s\n", value.len, value.data);

  // execute commands
  expr.len = sprintf( expr.data, "s a=123,b=456");
  ret = MNMExecute( Connect, &expr);
  if( !ret)
  {
    HandleMNMError( Connect);
    return 0;
  }

  // read 2 variables as list
  expr.len = sprintf( expr.data, "$lb(a,b)");
  ret = MNMRead( Connect, &expr, &value);
  if( !ret)
  {
    HandleMNMError( Connect);
    return 0;
  }

  // get list length
  int ListLength = MNMListLength( &value);
  printf( "List length: %d\n", ListLength);

  // unpack list structure
  MINIMSTR element;
  MNMListGet( &value, 1, &element);

  // show value
  printf( "First list item is: %.*s\n", element.len, element.data);

  MNMListGet( &value, 2, &element);
  
  printf( "Second list item is: %.*s\n", element.len, element.data);

  printf( "Example finished, press Enter.");
  fgetc( stdin);

  // disconnect from MiniM
  MNMConnectClose( Connect);

  // remove connect
  MNMDestroyConnect( Connect);

  return 0;
};
 