/*

    MiniM Server Connect

    Java to MiniM Server Connect example using JNI

*/

#include <memory.h>
#include <stdlib.h>
#include <stdio.h>
#include <stddef.h>

#include "minimsc.h"
#include "minimscj.h"

JNIEnv* current_env = NULL;
jobject current_object = 0;

static void STDCALL internal_callback( HMNMConnect pConnect,
  MINIMSTR* _Command, MINIMSTR* _Answer)
{
  // create Java data
  jbyteArray Command = current_env->NewByteArray( _Command->len);
  current_env->SetByteArrayRegion( Command, 0, _Command->len, (const jbyte*)_Command->data);

  // call current_env, Java object of class minimscj, function DoGroupRead
  jclass cls = current_env->GetObjectClass( current_object);
  jmethodID mid = current_env->GetMethodID( cls, "DoCallback", "([B)[B");
  if( mid == 0)
  {
    // nothing to call
    return;
  }

  jobject ret = current_env->CallObjectMethod( current_object, mid, Command);

  // decode jobject ret as jbyteArray and place data into _Answer
  jbyteArray b = (jbyteArray)ret;
  int len = current_env->GetArrayLength( b);
  jbyte* data = current_env->GetByteArrayElements( b, NULL);

  _Answer->len = len;

  memcpy( _Answer->data, data, len);

  current_env->ReleaseByteArrayElements( b, data, JNI_ABORT);


  return;
};

static void STDCALL internal_groupread( HMNMConnect pConnect,
  MINIMSTR* _Value)
{
  // create Java data
  jbyteArray Value = current_env->NewByteArray( _Value->len);
  current_env->SetByteArrayRegion( Value, 0, _Value->len, (const jbyte*)_Value->data);

  // call current_env, Java object of class minimscj, function DoGroupRead
  jclass cls = current_env->GetObjectClass( current_object);
  // pass function name and signature
  jmethodID mid = current_env->GetMethodID( cls, "DoGroupRead", "([B)V");
  if( mid == 0)
  {
    // nothing to call
    return;
  }

  current_env->CallVoidMethod( current_object, mid, Value);

  return;
};

static void STDCALL internal_output( HMNMConnect pConnect,
  MINIMSTR* _Value)
{
  // create Java data
  jbyteArray Value = current_env->NewByteArray( _Value->len);
  current_env->SetByteArrayRegion( Value, 0, _Value->len, (const jbyte*)_Value->data);

  // call current_env, Java object of class minimscj, function DoOutput
  jclass cls = current_env->GetObjectClass( current_object);
  // pass function name and signature
  jmethodID mid = current_env->GetMethodID( cls, "DoOutput", "([B)V");
  if( mid == 0)
  {
    // nothing to call
    return;
  }

  current_env->CallVoidMethod( current_object, mid, Value);

  return;
};

extern "C" JNIEXPORT jint JNICALL Java_minimscj_MNMText
  (JNIEnv* env, jobject, jbyteArray _source, jbyteArray _target)

{

  MINIMSTR Source;

  MINIMSTR Target;


  // transform Java data into MiniM data

  int len = env->GetArrayLength( _source);

  jbyte* data = env->GetByteArrayElements( _source, NULL);

  Source.len = len;

  memcpy( Source.data, data, len);

  env->ReleaseByteArrayElements( _source, data, JNI_ABORT);


  int ret = MNMText( &Source, &Target);

  if( !ret)

  {

    return -1;

  }


  // transform MiniM data to Java data

  env->SetByteArrayRegion( _target, 0, Target.len, (const jbyte*)Target.data);


  return Target.len;

};


extern "C" JNIEXPORT jint JNICALL Java_minimscj_MNMExecuteOutput
  (JNIEnv* env, jobject _object, jlong _Connect, jbyteArray _Commands)

{

  current_env = env;

  current_object = _object;

  MNMSetOutput( (HMNMConnect)(ptrdiff_t)_Connect, internal_output);


  MINIMSTR Commands;


  // transform Java data into MiniM data

  Commands.len = env->GetArrayLength( _Commands);

  jbyte* data = env->GetByteArrayElements( _Commands, NULL);

  memcpy( Commands.data, data, Commands.len);

  env->ReleaseByteArrayElements( _Commands, data, JNI_ABORT);


  return MNMExecuteOutput( (HMNMConnect)(ptrdiff_t)_Connect, &Commands);

};


extern "C" JNIEXPORT jint JNICALL Java_minimscj_MNMKill
  (JNIEnv* env, jobject _object, jlong _Connect, jbyteArray _VarName)

{

  current_env = env;

  current_object = _object;

  MINIMSTR VarName;


  int len = env->GetArrayLength( _VarName);

  jbyte* data = env->GetByteArrayElements( _VarName, NULL);

  VarName.len = len;

  memcpy( VarName.data, data, len);

  env->ReleaseByteArrayElements( _VarName, data, JNI_ABORT);


  return MNMKill( (HMNMConnect)(ptrdiff_t)_Connect, &VarName);

};


extern "C" JNIEXPORT jint JNICALL Java_minimscj_MNMWrite
  (JNIEnv* env, jobject _object, jlong _Connect,

    jbyteArray _VarName, jbyteArray _VarValue)

{

  current_env = env;

  current_object = _object;

  MINIMSTR VarName;

  MINIMSTR VarValue;


  int len = env->GetArrayLength( _VarName);

  jbyte* data = env->GetByteArrayElements( _VarName, NULL);

  VarName.len = len;

  memcpy( VarName.data, data, len);

  env->ReleaseByteArrayElements( _VarName, data, JNI_ABORT);


  len = env->GetArrayLength( _VarValue);

  data = env->GetByteArrayElements( _VarValue, NULL);

  VarValue.len = len;

  memcpy( VarValue.data, data, len);

  env->ReleaseByteArrayElements( _VarValue, data, JNI_ABORT);


  return MNMWrite( (HMNMConnect)(ptrdiff_t)_Connect, &VarName, &VarValue);

};


extern "C" JNIEXPORT jint JNICALL Java_minimscj_MNMListLength
  (JNIEnv* env, jobject, jbyteArray _List)

{

  MINIMSTR List;

  int len = env->GetArrayLength( _List);

  jbyte* data = env->GetByteArrayElements( _List, NULL);

  List.len = len;

  memcpy( List.data, data, len);

  env->ReleaseByteArrayElements( _List, data, JNI_ABORT);

  return MNMListLength( &List);

};


extern "C" JNIEXPORT jint JNICALL Java_minimscj_MNMListSet
  (JNIEnv* env, jobject, jbyteArray _List, jint _Pos,

    jbyteArray _Element, jbyteArray _Result)

{

  MINIMSTR List;

  MINIMSTR Element;


  int len = env->GetArrayLength( _List);

  jbyte* data = env->GetByteArrayElements( _List, NULL);

  List.len = len;

  memcpy( List.data, data, len);

  env->ReleaseByteArrayElements( _List, data, JNI_ABORT);


  len = env->GetArrayLength( _Element);

  data = env->GetByteArrayElements( _Element, NULL);

  Element.len = len;

  memcpy( Element.data, data, len);

  env->ReleaseByteArrayElements( _Element, data, JNI_ABORT);


  int result = MNMListSet( &List, _Pos, &Element);


  if( !result)

  {

    return -1;

  }


  env->SetByteArrayRegion( _Result, 0, List.len, (const jbyte*)List.data);


  return List.len;

};


extern "C" JNIEXPORT jint JNICALL Java_minimscj_MNMListGet
  (JNIEnv* env, jobject, jbyteArray _List, jint _Pos, jbyteArray _Res)

{

  MINIMSTR List;

  MINIMSTR Element;


  // transform Java data into MiniM data

  List.len = env->GetArrayLength( _List);

  jbyte* data = env->GetByteArrayElements( _List, NULL);

  memcpy( List.data, data, List.len);

  env->ReleaseByteArrayElements( _List, data, JNI_ABORT);


  int ret = MNMListGet( &List, _Pos, &Element);


  if( !ret)

  {

    return -1;

  }


  // transform MiniM data to Java data

  env->SetByteArrayRegion( _Res, 0, Element.len, (const jbyte*)Element.data);

  return Element.len;

};


extern "C" JNIEXPORT jint JNICALL Java_minimscj_MNMExecute
  (JNIEnv* env, jobject _object, jlong _Connect, jbyteArray _Commands)

{

  current_env = env;

  current_object = _object;


  MINIMSTR Commands;


  // transform Java data into MiniM data

  Commands.len = env->GetArrayLength( _Commands);

  jbyte* data = env->GetByteArrayElements( _Commands, NULL);

  memcpy( Commands.data, data, Commands.len);

  env->ReleaseByteArrayElements( _Commands, data, JNI_ABORT);


  return MNMExecute( (HMNMConnect)(ptrdiff_t)_Connect, &Commands);

};


extern "C" JNIEXPORT jint JNICALL Java_minimscj_MNMRead
  (JNIEnv* env, jobject _object, jlong _Connect,

   jbyteArray _Expr, jbyteArray _Value)

{
  current_env = env;

  current_object = _object;


  MINIMSTR expr;

  MINIMSTR value;


  // transform Java data into MiniM data

  jint len = env->GetArrayLength( _Expr);

  expr.len = len;

  jbyte* data = env->GetByteArrayElements( _Expr, NULL);

  memcpy( expr.data, data, len);

  env->ReleaseByteArrayElements( _Expr, data, JNI_ABORT);


  jint ret = MNMRead( (HMNMConnect)(ptrdiff_t)_Connect, &expr, &value);


  if( ret)

  {

    // transform MiniM data to Java data

    env->SetByteArrayRegion( _Value, 0, value.len, (const jbyte*)value.data);

    return value.len;

  }

  return -1;

};


extern "C" JNIEXPORT jbyteArray JNICALL Java_minimscj_MNMGetLastError
  (JNIEnv* env, jobject, jlong _Connect)

{

  MINIMSTR err;

  MNMGetLastError( (HMNMConnect)(ptrdiff_t)_Connect, &err);


  jbyteArray ret = env->NewByteArray( err.len);

  env->SetByteArrayRegion( ret, 0, err.len, (const jbyte*)err.data);


  return ret;

};


extern "C" JNIEXPORT void JNICALL Java_minimscj_MNMDestroyConnect
  (JNIEnv *, jobject, jlong _Connect)

{

  MNMDestroyConnect( (HMNMConnect)(ptrdiff_t)_Connect);

  return;

};


extern "C" JNIEXPORT jint JNICALL Java_minimscj_MNMConnectClose
  (JNIEnv * _env, jobject, jlong _Connect)

{

  return MNMConnectClose( (HMNMConnect)(ptrdiff_t)_Connect);

};


extern "C" JNIEXPORT jint JNICALL Java_minimscj_MNMConnectOpen
  (JNIEnv * env, jobject _object, jlong _Connect)

{
   current_env = env;

  current_object = _object;

  return MNMConnectOpen( (HMNMConnect)(ptrdiff_t)_Connect);

};


extern "C" JNIEXPORT jlong JNICALL Java_minimscj_MNMCreateConnect
  (JNIEnv* env, jobject, jstring _Server, jint _Port, jstring _Database)

{

  char* Server = (char*)env->GetStringUTFChars( _Server, NULL);

  char* Database = (char*)env->GetStringUTFChars( _Database, NULL);


  jlong ret = (jlong)(ptrdiff_t)MNMCreateConnect( Server, _Port, Database);


  if( ret)

  {

    MNMSetGroupRead( (HMNMConnect)(ptrdiff_t)ret, internal_groupread);

    MNMSetCallback( (HMNMConnect)(ptrdiff_t)ret, internal_callback);

  }


  env->ReleaseStringUTFChars( _Server, Server);

  env->ReleaseStringUTFChars( _Database, Database);

  

  return ret;

};


