/*

	MiniM Server Connect example.

	JNI class to illustrate how to call 
	MiniM Server Connect from Java.

*/

import java.io.File;

public class minimscj
{
	public String CodePage;
	public String Server;
	public int Port;
	public String Database;
	private long MNMConnect;
	public final int MINIMSTR_MAX = 32 * 1024 - 1;
	private String strMaxString = "Max string exeeded.";

	// this interface need to be inherited in application class to use output data
	public interface Output 
	{
		public void Output( minimscj Connect, String Value);
	};

	public Output callback = null;

	// save new output notifier
	void SetOutput( Output cb)
	{
		callback = cb;
		return;
	};

	// this function to call from JNI dll
	public void DoOutput( byte _Value[])
	{
		if( callback != null)
		{
			try
			{
				String Value = new String( _Value, CodePage);
				callback.Output( this, Value);
			}
			catch( java.io.UnsupportedEncodingException e)
			{
				e.printStackTrace();
			}
		}
		return;
	};

	// this interface need to be inherited in application class to use output data
	public interface GroupRead 
	{
		public void GroupRead( minimscj Connect, String Value);
	};

	public GroupRead GroupReadFunc = null;

	// save new group read notifier
	public void SetGroupRead( GroupRead func)
	{
		GroupReadFunc = func;
		return;
	};

	// this function to call from JNI dll
	public void DoGroupRead( byte _Value[])
	{
		if( GroupReadFunc != null)
		{
			try
			{
				String Value = new String( _Value, CodePage);
				GroupReadFunc.GroupRead( this, Value);
			}
			catch( java.io.UnsupportedEncodingException e)
			{
				e.printStackTrace();
			}
		}
		return;
	};

	// this interface need to be inherited in 
	// application class to use callback functionality
	public interface Callback
	{
		public String Callback( minimscj Connect, String Command);
	};

	public Callback CallbackFunc = null;

	// save new callback handler
	public void SetCallback( Callback func)
	{
		CallbackFunc = func;
		return;
	};

	public byte[] DoCallback( byte _Command[])
		throws java.io.UnsupportedEncodingException
	{
		if( CallbackFunc != null)
		{
			String Command = new String( _Command, CodePage);
			String ret = CallbackFunc.Callback( this, Command);
			return ret.getBytes( CodePage);
		}
		else
		{
			return new byte[ 0];
		}
	};

	static
	{
		String dllname = "minimscj.dll";
		try
		{
			// first see in PATH
			System.load(dllname);
		}
		catch( Throwable e1)
		{
			// failed, see in current directory
			try
			{
				File dir = new File( ".");
				System.load( dir.getCanonicalPath() + "\\" + dllname );
			}
			catch( Throwable e2)
			{
				e2.printStackTrace();
			}
		}
	}

	private native long MNMCreateConnect( String Server, int Port, String Database);

	public minimscj( String _Server, int _Port, String _Database, String _CodePage)
		throws java.lang.Exception
	{
		CodePage = _CodePage;
		Server = _Server;
		Port = _Port;
		Database = _Database;
		MNMConnect = MNMCreateConnect( Server, Port, Database);
		if( MNMConnect == 0)
		{
			throw new java.lang.Exception( "Failed to create connect to MiniM.");
		}
	};

	private native int MNMConnectOpen( long _Connect);

	public void ConnectOpen()
		throws java.lang.Exception
	{
		if( MNMConnectOpen( MNMConnect) == 0)
		{
			throw new java.lang.Exception( "Failed to open connect to MiniM.");
		}
	};

	private native int MNMConnectClose( long _Connect);

	public void ConnectClose()
		throws java.lang.Exception
	{
		if( MNMConnectClose( MNMConnect) == 0)
		{
			throw new java.lang.Exception( "Failed to close connect to MiniM.");
		}
	};

	private native void MNMDestroyConnect( long _Connect);

	public void DestroyConnect()
	{
		MNMDestroyConnect( MNMConnect);
	};

	private native byte[] MNMGetLastError( long _Connect);

	public String GetLastError()
		throws java.io.UnsupportedEncodingException
	{
		byte b[] = MNMGetLastError( MNMConnect);
		return new String( b, CodePage);
	};

	private native int MNMRead( long _Connect, byte Value[], byte Res[]);

	public String Read( String _Value)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		if( _Value.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}
		byte[] ToRead = _Value.getBytes( CodePage);
		byte[] res = new byte[ MINIMSTR_MAX];
		int ret = MNMRead( MNMConnect, ToRead, res);
		if( ret < 0)
		{
			throw new java.lang.Exception( 
				"MiniM read error: " + GetLastError());
		}
		return new String( res, 0, ret, CodePage);
	};

	private native int MNMExecute( long _Connect, byte Commands[]);

	public void Execute( String _Commands)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		if( _Commands.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}
		byte[] Commands = _Commands.getBytes( CodePage);
		int ret = MNMExecute( MNMConnect, Commands);
		if( ret == 0)
		{
			throw new java.lang.Exception( 
				"MiniM exec error: " + GetLastError());
		}
		return;
	};

	private native int MNMExecuteOutput( long _Connect, byte Commands[]);

	public void ExecuteOutput( String _Commands)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		if( _Commands.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}
		byte[] Commands = _Commands.getBytes( CodePage);
		int ret = MNMExecuteOutput( MNMConnect, Commands);
		if( ret == 0)
		{
			throw new java.lang.Exception( 
				"MiniM exec error: " + GetLastError());
		}
		return;
	};

	private native int MNMText( byte _Source[], byte _Target[]);

	public String Text( String Source)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		if( Source.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}
		byte[] source = Source.getBytes( CodePage);
		byte[] res = new byte[ MINIMSTR_MAX];
		int ret = MNMText( source, res);
		if( ret < 0)
		{
			throw new java.lang.Exception( strMaxString);
		}
		return new String( res, 0, ret, CodePage);
	};

	private native int MNMListLength( byte _List[]);
	public int ListLength( String _List)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		if( _List.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}
		byte[] list = _List.getBytes( CodePage);
		return MNMListLength( list);
	};

	private native int MNMListGet( byte _List[], int Pos, byte Value[]);

	public String ListGet( String _List, int Pos)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		if( _List.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}
		byte[] list = _List.getBytes( CodePage);
		byte[] res = new byte[ MINIMSTR_MAX];
		int ret = MNMListGet( list, Pos, res);
		if( ret < 0)
		{
			throw new java.lang.Exception( "MiniM list unpack error.");
		}
		return new String( res, 0, ret, CodePage);
	};

	private native int MNMListSet( byte _List[], int Pos, byte Value[], byte Res[]);

	public String ListSet( String _List, int Pos, String _Element)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		if( _List.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}
		if( _Element.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}

		byte[] list = _List.getBytes( CodePage);
		byte[] element = _Element.getBytes( CodePage);
		byte[] res = new byte[ MINIMSTR_MAX];
		
		int ret = MNMListSet( list, Pos, element, res);
		if( ret < 0)
		{
			throw new java.lang.Exception( "MiniM list unpack error.");
		}

		return new String( res, 0, ret, CodePage);
	};

	private native int MNMWrite( long Connect, byte VarName[], byte VarValue[]);

	public void Write( String _VarName, String _VarValue)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		if( _VarName.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}
		if( _VarValue.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}
		byte[] VarName = _VarName.getBytes( CodePage);
		byte[] VarValue = _VarValue.getBytes( CodePage);

		int ret = MNMWrite( MNMConnect, VarName, VarValue);
		if( ret == 0)
		{
			throw new java.lang.Exception( 
				"MiniM write error: " + GetLastError());
		}
		return;
	};

	private native int MNMKill( long Connect, byte VarName[]);

	public void Kill( String _VarName)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		if( _VarName.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}

		byte[] VarName = _VarName.getBytes( CodePage);

		int ret = MNMKill( MNMConnect, VarName);
		if( ret == 0)
		{
			throw new java.lang.Exception( 
				"MiniM kill error: " + GetLastError());
		}
		return;
	};

};