/*

    MiniM

    C++ example to call MiniM Server Connect

*/

#include <stdlib.h>
#include <stdio.h>

#include "minimsc.h"

void HandleMNMError( HMNMConnect Connect)
{
  MINIMSTR err;
  MNMGetLastError( Connect, &err);
  printf( "MiniM error:\n%.*s\n", err.len, err.data);
  return;
};

// callback handler
void STDCALL CallbackHandler( HMNMConnect pConnect,
  MINIMSTR* Command, MINIMSTR* Answer)
{
  // in this contex we can call MiniM
  // in example 5 we ignore Command value
  MINIMSTR Expression;
  MINIMSTR Value;
  Expression.len = sprintf( (char*)Expression.data, "%s", "$zt($p($h,\",\",2))");
  MNMRead( pConnect, &Expression, &Value);

  // compose callback answer
  Answer->len = sprintf( (char*)Answer->data, " time: %.*s", Value.len, Value.data);

  // trace execution order
  printf( "Client on command \"%.*s\" return: %.*s\n",
    Command->len, Command->data, Answer->len, Answer->data);

  return;
};

int main(int argc, char* argv[])
{
  printf( "C++ example how to call MiniM Server Connect\n");

  // create connect to MiniM
  HMNMConnect Connect = MNMCreateConnect( (char*)"192.168.121.1", 5000, (char*)"user");

  // activate connection
  int ret = MNMConnectOpen( Connect);
  if( !ret)
  {
    HandleMNMError( Connect);
    return 0;
  }

  // set callback handler
  MNMSetCallback( Connect, CallbackHandler);

  // call routine which call client back
  MINIMSTR Expression;
  Expression.len = sprintf( (char*)Expression.data, "%s", "$$callback^example5()");
  MINIMSTR Value;
  MNMRead( Connect, &Expression, &Value);

  // trace execution order
  printf( "Server return: %.*s\n", Value.len, Value.data);

  MNMSetCallback( Connect, NULL);

  printf( "Example finished, press Enter.");
  fgetc( stdin);

  // disconnect from MiniM
  MNMConnectClose( Connect);

  // remove connect
  MNMDestroyConnect( Connect);

  return 0;
};
 