/*

    MiniM

    MiniM Server Connect
    DLL interface

    Copyright (C) Eugene Karataev
    All rights reserved

*/

#ifndef __MINIMSC_H__
#define __MINIMSC_H__

#ifdef __cplusplus
extern "C" {
#endif

/* max string size */
#define MINIMSTR_MAX ( 32 * 1024 )

#pragma pack(push,1)
typedef struct _MINIMSTR
{
  unsigned short len;
  unsigned char data[ MINIMSTR_MAX];
} MINIMSTR;
#pragma pack(pop)

typedef void* HMNMConnect;

#ifndef MNMSCPROC
#define MNMSCPROC __stdcall 
#endif

/* create connect object */
HMNMConnect MNMSCPROC MNMCreateConnect( char* server,
  int port, char* database);

/* destroy connect object */
void MNMSCPROC MNMDestroyConnect( HMNMConnect pConnect);

/* connect to MiniM server */
int MNMSCPROC MNMConnectOpen( HMNMConnect pConnect);

/* disconnect from MiniM server */
int MNMSCPROC MNMConnectClose( HMNMConnect pConnect);

/* get last error description */
void MNMSCPROC MNMGetLastError( HMNMConnect pConnect, MINIMSTR* pError);

/* read M expression */
int MNMSCPROC MNMRead( HMNMConnect pConnect, MINIMSTR* Expression,
  MINIMSTR* Result);

/* write M variable */
int MNMSCPROC MNMWrite( HMNMConnect pConnect, MINIMSTR* VarName,
  MINIMSTR* VarValue);

/* execute commands */
int MNMSCPROC MNMExecute( HMNMConnect pConnect, MINIMSTR* Commands);

/* kill variable */
int MNMSCPROC MNMKill( HMNMConnect pConnect, MINIMSTR* VarName);

/* type of procedure to intercept output */
typedef void ( __stdcall* mnmoutputproc_t )( HMNMConnect pConnect,
  MINIMSTR* Value );

/* set procedure to intercept output */
void MNMSCPROC MNMSetOutput( HMNMConnect pConnect, mnmoutputproc_t pProc);

/* execute commands with output interception */
int MNMSCPROC MNMExecuteOutput( HMNMConnect pConnect, MINIMSTR* Commands);

/* type of procedure to accept group read data */
typedef void ( __stdcall* mnmgroupreadproc_t)( HMNMConnect pConnect,
  MINIMSTR* Value);

void MNMSCPROC MNMSetGroupRead( HMNMConnect pConnect, mnmgroupreadproc_t pProc);

/* type of procedure ho handle callback call */
typedef void ( __stdcall* mnmcallbackproc_t)( HMNMConnect pConnect,
  MINIMSTR* Command, MINIMSTR* Answer);

void MNMSCPROC MNMSetCallback( HMNMConnect pConnect,
  mnmcallbackproc_t pProc);

/* get list element as $listget(list,pos) */
int MNMSCPROC MNMListGet( MINIMSTR* List, int pos, MINIMSTR* Element);

/* set list element as s $list(list,pos)=element */
int MNMSCPROC MNMListSet( MINIMSTR* List, int pos, MINIMSTR* Element);

/* return count of list elements */
int MNMSCPROC MNMListLength( MINIMSTR* List);

/*
    decorate '"' if need

    "abcd"      ->   "abcd"
    "ab\"cd"    ->   "ab""cd"
    "ab\r\ncd"  ->   "ab"_$C(13,10)_"cd"

    return 0 if Target exeed MINIMSTR limit
*/
int MNMSCPROC MNMText( MINIMSTR* Source, MINIMSTR* Target);

#ifdef __cplusplus
}
#endif

/* __MINIMSC_H__ */
#endif
