﻿'
' 
' MiniM
' VB.NET example how to call MiniM Server Connect
' 
' This demo class MiniMExamples.MiniMSC 
' demonstrates how to declare and call minimsc.dll functions in VB.NET
'
' 
' Demo only. To use minimsc.dll inside your project write class 
' using your project coding and error handling conventions.
' 
' 
Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Runtime.InteropServices

' Namespace MiniMExamples
Class MiniMSC
    <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Ansi, Pack:=1)> _
    Public Structure MINIMSTR
        Public Const MINIMSTR_MAX As Integer = 32 * 1024 - 1
        Public len As Short
        <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=MINIMSTR_MAX)> _
        Public data As String
    End Structure

    Public Delegate Sub OutputProc(ByVal MNMConnect As IntPtr, ByRef Value As MINIMSTR)

    Public Delegate Sub GroupReadProc(ByVal MNMConnect As IntPtr, ByRef Value As MINIMSTR)

    Public Delegate Sub CallbackProc(ByVal MNMConnect As IntPtr, ByRef Command As MINIMSTR, ByRef Answer As MINIMSTR)

    Public Shared Sub StringToMINIMSTR(ByVal s As String, ByRef str As MINIMSTR)
        If s.Length <= MINIMSTR.MINIMSTR_MAX Then
            str.len = CShort(s.Length)
            str.data = s
        Else
            str.len = CShort(MINIMSTR.MINIMSTR_MAX)
            str.data = s.Substring(0, str.len)
        End If
        Exit Sub
    End Sub

    Public Shared Function MINIMSTRToString(ByRef str As MINIMSTR) As String
        Dim s As New StringBuilder()
        s.Append(str.data, 0, str.len)
        Return s.ToString()
    End Function

    Public Const DefaultPort As Integer = 5000
    Public Const DefaultDatabase As String = "USER"
    Public Const DefaultServer As String = "localhost"

    Public Port As Integer
    Public Server As String
    Public Database As String

    Protected MNMConnect As IntPtr
    Protected Proc As OutputProc
    Protected grProc As GroupReadProc
    Protected cbProc As CallbackProc

    Public Sub New()
        Port = DefaultPort
        Server = DefaultServer
        Database = DefaultDatabase
        MNMConnect = 0
        Proc = Nothing
        grProc = Nothing
        cbProc = Nothing
    End Sub

    <DllImport("minimsc.dll", CallingConvention:=CallingConvention.StdCall, EntryPoint:="MNMCreateConnect", ExactSpelling:=True, CharSet:=CharSet.Ansi)> _
    Public Shared Function MNMCreateConnect(ByVal server As String, ByVal port As Integer, ByVal database As String) As IntPtr
    End Function

    Public Function CreateConnect() As Boolean
        MNMConnect = MNMCreateConnect(Server, Port, Database)
        Return MNMConnect <> 0
    End Function

    <DllImport("minimsc.dll", CallingConvention:=CallingConvention.StdCall, EntryPoint:="MNMConnectOpen", ExactSpelling:=True, CharSet:=CharSet.Ansi)> _
    Public Shared Function MNMConnectOpen(ByVal Connect As IntPtr) As Integer
    End Function

    Public Function ConnectOpen() As Boolean
        Return MNMConnectOpen(MNMConnect) <> 0
    End Function

    <DllImport("minimsc.dll", CallingConvention:=CallingConvention.StdCall, EntryPoint:="MNMGetLastError", ExactSpelling:=True, CharSet:=CharSet.Ansi)> _
    Public Shared Sub MNMGetLastError(ByVal Connect As IntPtr, ByRef pError As MINIMSTR)
    End Sub

    Public Function GetLastError() As String
        Dim [Error] As New MINIMSTR()
        MNMGetLastError(MNMConnect, [Error])
        Return MINIMSTRToString([Error])
    End Function

    <DllImport("minimsc.dll", CallingConvention:=CallingConvention.StdCall, EntryPoint:="MNMRead", ExactSpelling:=True, CharSet:=CharSet.Ansi)> _
    Public Shared Function MNMRead(ByVal Connect As IntPtr, ByRef Expression As MINIMSTR, ByRef Result As MINIMSTR) As Integer
    End Function

    Public Function Read(ByRef Expression As MINIMSTR, ByRef Result As MINIMSTR) As Integer
        Return MNMRead(MNMConnect, Expression, Result)
    End Function

    <DllImport("minimsc.dll", CallingConvention:=CallingConvention.StdCall, EntryPoint:="MNMExecute", ExactSpelling:=True, CharSet:=CharSet.Ansi)> _
    Public Shared Function MNMExecute(ByVal Connect As IntPtr, ByRef Commands As MINIMSTR) As Integer
    End Function

    Public Function Execute(ByRef Commands As MINIMSTR) As Integer
        Return MNMExecute(MNMConnect, Commands)
    End Function

    <DllImport("minimsc.dll", CallingConvention:=CallingConvention.StdCall, EntryPoint:="MNMListLength", ExactSpelling:=True, CharSet:=CharSet.Ansi)> _
    Public Shared Function MNMListLength(ByRef List As MINIMSTR) As Integer
    End Function

    <DllImport("minimsc.dll", CallingConvention:=CallingConvention.StdCall, EntryPoint:="MNMListGet", ExactSpelling:=True, CharSet:=CharSet.Ansi)> _
    Public Shared Function MNMListGet(ByRef List As MINIMSTR, ByVal pos As Integer, ByRef Element As MINIMSTR) As Integer
    End Function
    <DllImport("minimsc.dll", CallingConvention:=CallingConvention.StdCall, EntryPoint:="MNMListSet", ExactSpelling:=True, CharSet:=CharSet.Ansi)> _
    Public Shared Function MNMListSet(ByRef List As MINIMSTR, ByVal pos As Integer, ByRef Element As MINIMSTR) As Integer
    End Function
    <DllImport("minimsc.dll", CallingConvention:=CallingConvention.StdCall, EntryPoint:="MNMText", ExactSpelling:=True, CharSet:=CharSet.Ansi)> _
    Public Shared Function MNMText(ByRef Source As MINIMSTR, ByRef Target As MINIMSTR) As Integer
    End Function

    <DllImport("minimsc.dll", CallingConvention:=CallingConvention.StdCall, EntryPoint:="MNMConnectClose", ExactSpelling:=True, CharSet:=CharSet.Ansi)> _
    Public Shared Function MNMConnectClose(ByVal Connect As IntPtr) As Integer
    End Function

    Public Function ConnectClose() As Integer
        Dim ret As Integer = MNMConnectClose(MNMConnect)
        Return ret
    End Function

    <DllImport("minimsc.dll", CallingConvention:=CallingConvention.StdCall, EntryPoint:="MNMDestroyConnect", ExactSpelling:=True, CharSet:=CharSet.Ansi)> _
    Public Shared Sub MNMDestroyConnect(ByVal Connect As IntPtr)
    End Sub

    Public Sub DestroyConnect()
        MNMDestroyConnect(MNMConnect)
        Exit Sub
    End Sub

    <DllImport("minimsc.dll", CallingConvention:=CallingConvention.StdCall, EntryPoint:="MNMSetCallback", ExactSpelling:=True, CharSet:=CharSet.Ansi)> _
    Public Shared Sub MNMSetCallback(ByVal Connect As IntPtr, ByVal Proc As CallbackProc)
    End Sub

    Public Sub SetCallback(ByVal _Proc As CallbackProc)
        cbProc = _Proc
        MNMSetCallback(MNMConnect, cbProc)
        Exit Sub
    End Sub

    <DllImport("minimsc.dll", CallingConvention:=CallingConvention.StdCall, EntryPoint:="MNMSetGroupRead", ExactSpelling:=True, CharSet:=CharSet.Ansi)> _
    Public Shared Sub MNMSetGroupRead(ByVal Connect As IntPtr, ByVal Proc As GroupReadProc)
    End Sub

    Public Sub SetGroupRead(ByVal _Proc As GroupReadProc)
        grProc = _Proc
        MNMSetGroupRead(MNMConnect, grProc)
        Exit Sub
    End Sub

    <DllImport("minimsc.dll", CallingConvention:=CallingConvention.StdCall, EntryPoint:="MNMSetOutput", ExactSpelling:=True, CharSet:=CharSet.Ansi)> _
    Public Shared Sub MNMSetOutput(ByVal Connect As IntPtr, ByVal Proc As OutputProc)
    End Sub

    Public Sub SetOutput(ByVal _Proc As OutputProc)
        Proc = _Proc
        MNMSetOutput(MNMConnect, Proc)
        Exit Sub
    End Sub

    <DllImport("minimsc.dll", CallingConvention:=CallingConvention.StdCall, EntryPoint:="MNMExecuteOutput", ExactSpelling:=True, CharSet:=CharSet.Ansi)> _
    Public Shared Function MNMExecuteOutput(ByVal Connect As IntPtr, ByRef Commands As MINIMSTR) As Integer
    End Function

    Public Function ExecuteOutput(ByRef Commands As MINIMSTR) As Integer
        Return MNMExecuteOutput(MNMConnect, Commands)
    End Function

    <DllImport("minimsc.dll", CallingConvention:=CallingConvention.StdCall, EntryPoint:="MNMWrite", ExactSpelling:=True, CharSet:=CharSet.Ansi)> _
    Public Shared Function MNMWrite(ByVal Connect As IntPtr, ByRef VarName As MINIMSTR, ByRef VarValue As MINIMSTR) As Integer
    End Function

    Public Function Write(ByRef VarName As MINIMSTR, ByRef VarValue As MINIMSTR) As Integer
        Return MNMWrite(MNMConnect, VarName, VarValue)
    End Function

    <DllImport("minimsc.dll", CallingConvention:=CallingConvention.StdCall, EntryPoint:="MNMKill", ExactSpelling:=True, CharSet:=CharSet.Ansi)> _
    Public Shared Function MNMKill(ByVal Connect As IntPtr, ByRef VarName As MINIMSTR) As Integer
    End Function

    Public Function Kill(ByRef VarName As MINIMSTR) As Integer
        Return MNMKill(MNMConnect, VarName)
    End Function

End Class
' End Namespace